use std::str::FromStr;

use crate::botv2::{
    domain::concour::{
        check_if_allowed::check_if_allowed,
        set_periode::{set_periode, SetPeriodeConcourError},
    },
    init::{Context, Error},
};
use chrono::{DateTime, Utc};
use cron::Schedule;
use poise::{
    serenity_prelude::{model::colour, CreateEmbed, CreateEmbedFooter, Mentionable, RoleId},
    CreateReply,
};
use tracing::{info, instrument, warn};

/// Update the step duration of a concour (only for admin)
#[instrument(skip(ctx), level = "info", fields(channel = ctx.channel_id().get(), guild = ?ctx.guild_id().unwrap().get()))]
#[poise::command(
    slash_command,
    prefix_command,
    category = "server_config",
    guild_only = true
)]
pub async fn period(
    ctx: Context<'_>,
    #[description = "Dureer d'une étape du concours au format CRON (min: 6H d'interval)"]
    cron_schedule: String,
) -> Result<(), Error> {
    let guild = match ctx.guild_id() {
        Some(guild) => guild,
        None => return Ok(()),
    };
    let entity_name = ctx.data().entity_name.clone();
    let footer = CreateEmbedFooter::new(entity_name.clone());
    match check_if_allowed(guild.get(), ctx.author().id.get(), ctx.http()).await {
        Ok(ok) => {
            if !ok {
                let embed = CreateEmbed::new()
                    .title("You are not an admin")
                    .color(colour::Color::RED)
                    .footer(footer);
                if let Err(why) = ctx
                    .send(CreateReply::default().embed(embed).ephemeral(true))
                    .await
                {
                    tracing::error!("Error sending message: {:?}", why);
                }
                return Ok(());
            }
        }
        Err(_) => {
            let embed = CreateEmbed::new()
                .title("You are not an admin")
                .color(colour::Color::RED)
                .footer(footer);
            if let Err(why) = ctx
                .send(CreateReply::default().embed(embed).ephemeral(true))
                .await
            {
                tracing::error!("Error sending message: {:?}", why);
            }
            return Ok(());
        }
    };
    let schedule = match Schedule::from_str(&cron_schedule) {
        Ok(schedule) => schedule,
        Err(err) => {
            warn!(err = err.to_string(), "Cron format is invalid");
            let embed = CreateEmbed::new()
                .title("Invalid CRON format")
                .color(colour::Color::RED)
                .footer(footer);
            if let Err(why) = ctx
                .send(CreateReply::default().embed(embed).ephemeral(true))
                .await
            {
                tracing::error!("Error sending message: {:?}", why);
            }
            return Ok(());
        }
    };
    let next_to_come = schedule.upcoming(Utc).take(10);
    // Check if the next_to_come has at least 6 hours of interval
    let mut last: Option<DateTime<Utc>> = None;
    for next in next_to_come {
        if let Some(last) = last {
            if next.timestamp() - last.timestamp() < 6 * 60 * 60 {
                let embed = CreateEmbed::new()
                    .title("Interval between steps is too short")
                    .description("Please provide a CRON schedule with at least 6 hours of interval")
                    .color(colour::Color::RED)
                    .footer(footer);
                if let Err(why) = ctx
                    .send(CreateReply::default().embed(embed).ephemeral(true))
                    .await
                {
                    tracing::error!("Error sending message: {:?}", why);
                }
                warn!(
                    cron_schedule = cron_schedule,
                    "Interval between steps is too short"
                );
                return Ok(());
            }
        }
        last = Some(next);
    }
    info!(
        cron_schedule = cron_schedule.as_str(),
        "Setting concour periode"
    );
    let concour = match set_periode(guild.get(), ctx.channel_id().get(), cron_schedule).await {
        Ok(concour) => {
            if concour.is_none() {
                CreateEmbed::new()
                    .title("No concour updated")
                    .color(colour::Color::RED)
            } else {
                let concour = concour.unwrap();
                let mut output = CreateEmbed::new()
                    .title(concour.title)
                    .description(concour.description)
                    .field("Start date", concour.start_date.to_string(), false)
                    .field("Periode", concour.periode.to_string(), false);

                if concour.role_recompense == 0 {
                    output = output.field("Role récompense", "Aucun", false);
                } else {
                    output = output.field(
                        "Role récompense",
                        RoleId::new(concour.role_recompense).mention().to_string(),
                        false,
                    );
                }
                output
            }
        }
        Err(err) => match err {
            SetPeriodeConcourError::DoesntExist => CreateEmbed::new()
                .title("Concour dont exist")
                .color(colour::Color::RED),
            _ => CreateEmbed::new()
                .title("Error while creating concour")
                .field("Please contact your administrator", "", false)
                .field("Your concour is possibly not initied correctly", "", false)
                .color(colour::Color::RED),
        },
    };

    let mut builder = CreateReply::default().ephemeral(true);
    builder = builder.embed(concour.footer(footer));
    if let Err(why) = ctx.send(builder).await {
        tracing::error!("Error sending message: {:?}", why);
    }
    Ok(())
}
