use crate::botv2::{
    cmd::server_config::{admin::admin, feature::feature, init_server::init},
    domain::server_config::{
        check_if_server_enable_and_admin::check_if_server_enable_and_user_admin,
        get_server_config::get_server_config,
    },
    init::{Context, Error},
};
use poise::{
    serenity_prelude::{model::colour, CreateEmbed, CreateEmbedFooter, RoleId},
    CreateReply,
};
use tracing::instrument;

/// Show server config (alias of info)
#[instrument(skip(ctx), level = "info", fields(channel = ctx.channel_id().get(), guild = ?ctx.guild_id().unwrap().get()))]
#[poise::command(
    slash_command,
    prefix_command,
    category = "server_config",
    subcommands("init", "info", "feature", "admin"),
    guild_only = true
)]
pub async fn server(
    ctx: Context<'_>,
    #[description = "Reponse cacher ?"] ephemeral: Option<bool>,
) -> Result<(), Error> {
    server_info(ctx, ephemeral).await
}

/// Show server config
#[instrument(skip(ctx), level = "info", fields(channel = ctx.channel_id().get(), guild = ?ctx.guild_id().unwrap().get()))]
#[poise::command(
    slash_command,
    prefix_command,
    category = "server_config",
    guild_only = true
)]
pub async fn info(
    ctx: Context<'_>,
    #[description = "Reponse cacher ?"] ephemeral: Option<bool>,
) -> Result<(), Error> {
    server_info(ctx, ephemeral).await
}

#[instrument(skip(ctx), level = "info")]
async fn server_info(ctx: Context<'_>, ephemeral: Option<bool>) -> Result<(), Error> {
    let guild = match ctx.guild_id() {
        Some(guild) => guild,
        None => return Ok(()),
    };
    let entity_name = ctx.data().entity_name.clone();
    let footer = CreateEmbedFooter::new(entity_name.clone());
    match check_if_server_enable_and_user_admin(guild.get(), ctx.author().id.get(), ctx.http())
        .await
    {
        Ok((ok, _)) => {
            if !ok {
                let embed = CreateEmbed::new()
                    .title("You are not an admin")
                    .color(colour::Color::RED)
                    .footer(footer);
                if let Err(why) = ctx
                    .send(CreateReply::default().embed(embed).ephemeral(true))
                    .await
                {
                    tracing::error!("Error sending message: {:?}", why);
                }
                return Ok(());
            }
        }
        Err(_) => {
            let embed = CreateEmbed::new()
                .title("Error while fetching server config")
                .field("Please contact your administrator", "", false)
                .field("Your config is possibly not initied", "", false)
                .color(colour::Color::RED)
                .footer(footer);
            if let Err(why) = ctx
                .send(CreateReply::default().embed(embed).ephemeral(true))
                .await
            {
                tracing::error!("Error sending message: {:?}", why);
            }
            return Ok(());
        }
    };
    let answer = match get_server_config(guild.get()).await {
        Ok(config) => match config {
            Some(server_config) => {
                let mut embed = CreateEmbed::new()
                    .title(format!(
                        "Server config found: {}",
                        ctx.guild().unwrap().name
                    ))
                    .field("Enabled", server_config.enable.to_string(), true)
                    .field(
                        "Feature Auto_meme",
                        server_config.auto_meme.to_string(),
                        true,
                    )
                    .field(
                        "Feature Concour",
                        server_config.auto_concour.to_string(),
                        true,
                    )
                    .color(colour::Color::DARK_GREEN);

                let role_list: Vec<String> = server_config
                    .admin_role
                    .iter()
                    .map(|role_id| {
                        ctx.guild().unwrap().roles[&RoleId::new(*role_id)]
                            .name
                            .clone()
                    })
                    .collect();
                if !role_list.is_empty() {
                    embed = embed.field("Admin role", role_list.join(", "), false);
                } else {
                    embed = embed.field("Admin role", "No role found", false);
                }
                embed
            }
            None => CreateEmbed::new()
                .title("Server config not found")
                .color(colour::Color::RED),
        },
        Err(_) => CreateEmbed::new()
            .title("Error fetching server config")
            .color(colour::Color::RED),
    };
    let mut builder = CreateReply::default().embed(answer.footer(footer));
    if ephemeral.unwrap_or(true) {
        builder = builder.ephemeral(true);
    }
    if let Err(why) = ctx.send(builder).await {
        tracing::error!("Error sending message: {:?}", why);
    }
    Ok(())
}
