use super::check_if_server_enable::{check_if_server_enable, CheckIfServerEnableError};
use crate::db::server_config::ServerConfig;
use poise::serenity_prelude::{self, RoleId, UserId};
use tracing::{info, instrument};

#[instrument(level = "info", skip(http))]
pub async fn check_if_server_enable_and_user_admin(
    server_id: u64,
    user_id: u64,
    http: &serenity_prelude::Http,
) -> Result<(bool, Option<ServerConfig>), CheckIfServerEnableError> {
    let server_config = match check_if_server_enable(server_id).await {
        Ok((status, config)) => {
            if !status {
                return Ok((false, None));
            }
            config.unwrap()
        }
        Err(err) => return Err(err),
    };
    let guild = match http.get_guild(server_id.into()).await {
        Ok(guild) => guild,
        Err(err) => {
            tracing::error!(error = err.to_string(), "Error getting guild");
            return Err(CheckIfServerEnableError::UnknownError(
                "Error getting guild".to_string(),
            ));
        }
    };
    if guild.owner_id.get() == user_id {
        info!("User is owner of the server");
        return Ok((true, Some(server_config)));
    }
    match guild.member(http, UserId::new(user_id)).await {
        Ok(member) => {
            info!(user_name = member.user.name, "Checking if user is admin");

            if let Some(perm) = member.permissions {
                info!(perm = perm.bits(), "Checking if user has admin permission");
                if perm.administrator() {
                    info!("User has administarator permission");
                    return Ok((true, Some(server_config)));
                }
            }
            if member
                .permissions
                .into_iter()
                .any(|perm| perm.administrator())
                || guild.member_permissions(&member).administrator()
            {
                info!("User has administarator permission");
                return Ok((true, Some(server_config)));
            }
            info!("Checking if user has admin role");

            Ok((
                server_config
                    .clone()
                    .admin_role
                    .into_iter()
                    .any(|role_id| member.roles.contains(&RoleId::new(role_id))),
                Some(server_config),
            ))
        }
        Err(err) => {
            tracing::error!(error = err.to_string(), "Error getting member");
            return Err(CheckIfServerEnableError::UnknownError(
                "Error getting member".to_string(),
            ));
        }
    }
}
